#!/bin/sh
scriptdir=`dirname $0` ; cd ${scriptdir} ; scriptdir=`pwd` ; cd - > /dev/null

partedbin=`sudo which parted`

# max image size 1GB
maxSize=1073741824

liveSize=0
freeSize=0
partSize=0
needSize=0
usedSize=0
imageFound=0

handledir() {
 d=$1
 b=`basename $d`
 l=`dirname $d`/`echo $b | tr A-Z a-z`

 if [ -e $d -a -e $l ]; then
   echo Moving $d to $l
   cp -apP ${d}/. ${l}/ && rm -rf ${d}/
 fi
}

createimg() {
  if [ $2 -gt 0 ]; then
    echo "Creating ip/$1.img"
    sudo sync
    sudo umount ${workdir}/mount

    ofs=1048576
    sze=`expr 1048576 + 1048576`

    # add 1% free space
    minFree=`expr ${maxSize} / 100`
    minSize=`expr $2 + ${minFree}`
    newSize=${minSize}

    # 16 MB steps
    newBls=`expr 16 '*' 1048576`
    newBlc=`expr ${newSize} / ${newBls}`
    newSize=`expr ${newBlc} '*' ${newBls}`
    if [ ${minSize} -gt ${newSize} ]; then
      newSize=`expr ${newSize} + ${newBls}`
    fi

    if [ ${newSize} -gt ${maxSize} ]; then
      newSize=${maxSize}
    fi

    #newSize=${maxSize}

    # 1073741823
    maxEnd=`expr ${newSize} - 1`
    # 1072693248
    maxSzl=`expr ${newSize} - ${ofs}`
    # 1024
    maxMBs=`expr ${newSize} / 1048576`

    echo "Creating with ${maxMBs} MB, start ${ofs} byte, end ${maxEnd} byte, size ${maxSzl} byte"

    dd if=/dev/zero of=${workdir}/$1.img count=0 bs=1048576 seek=${maxMBs}

    LANG=C $partedbin ${workdir}/$1.img <<EOF
unit b
mklabel msdos
mkpart primary fat32 ${ofs}B ${maxEnd}B
toggle 1 boot
print
quit
EOF

    LOOPDEV=`sudo losetup --find --show -o ${ofs} --sizelimit ${maxSzl} ${workdir}/$1.img`

    lbl=EMBEDDED
    uid=0963C8B6

    echo ID: $uid label: $lbl

    zeroMBs=`expr ${maxSzl} / 1048576`
    zeroMBs=`expr ${zeroMBs} - 2`
    sudo dd if=/dev/zero of=$LOOPDEV count=${zeroMBs} bs=1048576 seek=2

    zeroMBs=`expr ${zeroMBs} + 2`
    zeroBs=`expr ${zeroMBs} '*' 1048576`
    if [ ${maxSzl} -gt ${zeroBs} ]; then
      sudo dd if=/dev/zero of=$LOOPDEV count=`expr ${maxSzl} - ${zeroBs}` bs=1 seek=${zeroBs}
    fi

    sudo mkfs.vfat -F 32 -h 2048 -i $uid -n $lbl $LOOPDEV

    sudo losetup -d $LOOPDEV

    ofs=0
    sze=0
    while read n b e s t f z ; do
      ofs=`echo $b | tr -d B`
      sze=`echo $s | tr -d B`
    done <<< $(LANG=C $partedbin ${workdir}/$1.img unit b print 2>/dev/null | grep -m1 fat)

    echo ofs: $ofs sze: $sze

    cd ${scriptdir}/

    sudo mount -o loop,offset=$ofs,sizelimit=$sze ${workdir}/$1.img ${workdir}/mount
  fi
}

mountimg() {
  imageFound=0

  if [ ! -e ${workdir}/iso/ip/$1.img.gz ]; then
    imgsrc=$1
    usedSize=0

    cd ${workdir}/

    for f in `ls iso/ip/$1-*.rpm 2>/dev/null` ; do
      r=`basename $f`

      if false ; then
        echo "Converting ip/$r"
        fakeroot alien $f
        echo "Extracting ip/$r"
        dpkg-deb -x $1_*.deb $1-pkg
      else
        echo "Extracting ip/$r"
        mkdir $1-pkg
        cd $1-pkg
        rpm2cpio ../$f | fakeroot cpio -idm
        cd ..
      fi

      if [ -e $1-pkg/mnt/$1flash ]; then
        imgsrc=$1-pkg/mnt/$1flash
        sudo mount -o bind ${imgsrc} mount
        cd mount
        usedSize=`du -d0 -B1 | cut -f 1`
        cd ..

      elif [ -e $1-pkg/mnt/ipflash ]; then
        imgsrc=$1-pkg/mnt/ipflash
        sudo mount -o bind ${imgsrc} mount
        cd mount
        usedSize=`du -d0 -B1 | cut -f 1`
        cd ..

      else
        sudo mount -o bind $1-pkg mount
      fi

      imageFound=1
    done

    if [ $usedSize -gt 0 -a "X$2" != "X--no-image" ]; then
      cd ${scriptdir}/

      if [ $1 = gaius -a -e ${workdir}/GPE/sources/boot.wim ]; then
        echo "Merging $1 and GPE"

        mkdir -p ${workdir}/$1-GPE
        cp -ap --no-preserve=ownership ${workdir}/${imgsrc}/. ${workdir}/$1-GPE/
        cp -ap --no-preserve=ownership ${workdir}/GPE/.       ${workdir}/$1-GPE/
        sudo sync

        sudo umount ${workdir}/mount

        rm ${workdir}/iso/ip/GPE.img.gz
        rm ${workdir}/GPE.img
        sudo sync

        cd ${workdir}/

        imgsrc=$1-GPE
        sudo mount -o bind ${imgsrc} mount
        cd mount
        usedSize=`du -d0 -B1 | cut -f 1`
        cd ..
      fi

      needSize=`expr ${usedSize} + 5000000`

      echo "Needing ${needSize} bytes"
      cd ${scriptdir}/

      createimg $1 ${needSize}

      if [ -e ${workdir}/${imgsrc}/efi ]; then
        cd ${workdir}/${imgsrc}

        mkdir -p ./boot
        mkdir -p ./efi/boot

        handledir ./Boot

        handledir ./boot/Fonts
        handledir ./boot/Resources

        handledir ./efi/Boot
        handledir ./efi/Microsoft

        handledir ./efi/microsoft/Boot

        handledir ./efi/microsoft/boot/Fonts
        handledir ./efi/microsoft/boot/Resources

        cd ${scriptdir}/
      fi

      sudo cp -ap --no-preserve=ownership ${workdir}/${imgsrc}/. ${workdir}/mount/
      sudo sync
      sudo umount ${workdir}/mount

      echo "Creating ip/$1.img.gz"
      cat ${workdir}/$1.img | gzip -c > ${workdir}/iso/ip/$1.img.gz
      sudo sync

      sudo mount -o loop,offset=$ofs,sizelimit=$sze ${workdir}/$1.img ${workdir}/mount

      cd ${workdir}/mount/

      df -B1 .

      echo ""
    fi

    cd ${scriptdir}/

  else
    echo "Extracting ip/$1.img.gz"
    gzip -cd ${workdir}/iso/ip/$1.img.gz > ${workdir}/$1.img

    ofs=0
    sze=0
    LANG=C $partedbin ${workdir}/$1.img unit b print
    while read n b e s t f z ; do
      ofs=`echo $b | tr -d B`
      sze=`echo $s | tr -d B`

      #echo mount -o loop,ro,offset=$ofs,sizelimit=$sze ${workdir}/$1.img ${workdir}/mount
      sudo mount -o loop,offset=$ofs,sizelimit=$sze ${workdir}/$1.img ${workdir}/mount

      imageFound=1
    done <<< $(LANG=C $partedbin ${workdir}/$1.img unit b print 2>/dev/null | grep -m1 fat)
  fi
}

unpack() {
  mountimg $1

  if [ $imageFound != 0 ]; then
    mkdir ${workdir}/$1
    cp -apP ${workdir}/mount/. ${workdir}/$1/
    sudo umount ${workdir}/mount
  fi
}

HPIPDEBARCH=amd64
HPIPDEBVER=1.63
HPIPDEBNAM=hp-tools
HPIPDEBDIR=${scriptdir}/${workdir}/${HPIPDEBNAM}-deb/${HPIPDEBNAM}_${HPIPDEBVER}

simpledeb() {
  mkdir -p ${HPIPDEBDIR}
  mkdir    ${HPIPDEBDIR}/DEBIAN

  cat <<EOFDRUHPIP | sudo tee ${HPIPDEBDIR}/DEBIAN/control > /dev/null
Package: ${HPIPDEBNAM}
Architecture: ${HPIPDEBARCH}
Maintainer: None
Depends: libc6
Priority: optional
Version: ${HPIPDEBVER}
Description: ${HPIPDEBNAM}
EOFDRUHPIP

  cd ${scriptdir}/${workdir}
  dpkg-deb --build ${HPIPDEBDIR}
}


sudo apt-get update
sudo apt-get install bash bzip2 coreutils dosfstools fakeroot file genisoimage grep gzip lzma parted sdparm tar wget unzip xz-utils
# for grub-mkrescue
#sudo apt-get install grub-common grub-efi-amd64-bin xorriso
# for IP300 and later
#sudo apt-get install rpm2cpio

partedbin=`sudo which parted`

isofile=HPIP.iso
for f in `ls -tr HPIP*.iso IP*.iso 2>/dev/null` ; do
  isofile=$f
done

if [ ! -e ${isofile} ]; then
  isofile=HPIP164B.2017_0201.3.iso
  # Gen8
  #wget -N https://downloads.hpe.com/pub/softlib2/software1/cd/p1450150448/v108035/HPIP163.2016_0720.i192.227.iso
  wget -N https://downloads.hpe.com/pub/softlib2/software1/cd/p1450150448/v125952/HPIP164B.2017_0201.3.iso
  # Gen9
  #wget -N https://downloads.hpe.com/pub/softlib2/software1/cd/p1450150448/v109733/HPIP250.2016_1015.169.iso
  #wget -N https://downloads.hpe.com/pub/softlib2/software1/cd/p1450150448/v128888/HPIP261.2017_0518.11.iso
  # Gen10
  #wget -N https://downloads.hpe.com/pub/softlib2/software1/cd/p1450150448/v105991/IP300.2017_0606.382.iso
fi

[ ! -e ${isofile} ] && exit 1

workdir=${isofile/.iso/}

hpipver=${workdir}
hpipver=${hpipver/HPIP/}
hpipver=${hpipver/IP/}
hpipver=${hpipver:0:1}.${hpipver:1}

echo ""
echo Using ${workdir}

mkdir -p ${workdir}/mount
mkdir -p ${workdir}/tmp

echo "Extracting `basename ${isofile}`"
mkdir -p ${workdir}/iso
sudo mount -o loop,ro ${isofile} ${workdir}/mount
cp -apP ${workdir}/mount/. ${workdir}/iso/ 
sudo umount ${workdir}/mount

unpack GPE
unpack gaius
unpack vid

for f in `ls ${workdir}/iso/ip/*-*.rpm 2>/dev/null` ; do
  r=`basename $f`
  n=`echo $r | sed s/'-[0-9].*'/''/g`

  if [ ! -e ${workdir}/${n}-pkg ]; then
    unpack $n --no-image
  fi
done

echo "Extracting initrd.img"
mkdir -p ${workdir}/initrd
cd ${workdir}/initrd/
cat ${scriptdir}/${workdir}/iso/system/initrd.img | lzma -dc | cpio -id
fakeroot tar -czf ${scriptdir}/${workdir}/hp-ilo-nand.tar.gz bin/nand*

cd ${scriptdir}

gaius=${workdir}/gaius

if [ -e $gaius/boot/initrd.img ]; then
  echo "Found gaius/boot/initrd.img"

  mkdir -p ${workdir}/gaius-initrd
  cd ${workdir}/gaius-initrd/
  cat ${scriptdir}/${gaius}/boot/initrd.img | lzma -dc | cpio -id

  cd ${scriptdir}
fi

if [ -e $gaius/system/squashfs.img ]; then
  echo "Found gaius/system/squashfs.img"
  sudo mount -o loop,ro $gaius/system/squashfs.img ${workdir}/mount

  cd ${workdir}/mount

  sudo bash -c 'tar czf '${scriptdir}/${workdir}'/hp-phoenix.tar.gz \
    bin/copylogs* \
    bin/hpblob* \
    opt/hp/hp-phoenix/srv/www/bin \
    opt/hp/hp-phoenix/srv/www/cfg \
    opt/hp/hp-phoenix/srv/www/py'
  sudo chmod ugo+r ${scriptdir}/${workdir}/hp-phoenix.tar.gz

  sudo bash -c 'cd usr/lib64/python2.7/site-packages ; tar czf '${scriptdir}/${workdir}'/hp-phoenix-python.tar.gz \
    hp*'
  sudo chmod ugo+r ${scriptdir}/${workdir}/hp-phoenix-python.tar.gz

  cd ${scriptdir}

  cd ${workdir}/mount

  sudo bash -c 'tar czf '${scriptdir}/${workdir}'/hp-scripting-tools.tar.gz \
    bin/conrep bin/hp-conrep bin/hpdiscovery bin/hp-discovery bin/hpev bin/hp-ev \
    bin/hp-hwquery bin/hp-ifhw bin/hp-lo100cfg bin/hp-rbsureset \
    bin/hprcu bin/hp-rcu bin/hp-reboot bin/hp-setbootorder bin/hp-statemgr \
    bin/hwquery bin/ifhw bin/lo100cfg bin/rbsureset bin/setbootorder bin/statemgr \
    opt/hp/hp-scripting-tools'
  sudo chmod ugo+r ${scriptdir}/${workdir}/hp-scripting-tools.tar.gz

  HPIPDEBVER=${hpipver/_/-}

  HPIPDEBNAM=hp-phoenix
  HPIPDEBDIR=${scriptdir}/${workdir}/${HPIPDEBNAM}-deb/${HPIPDEBNAM}_${HPIPDEBVER}_${HPIPDEBARCH}

  mkdir -p ${HPIPDEBDIR}

  cd ${HPIPDEBDIR}
  fakeroot tar xzf ${scriptdir}/${workdir}/hp-phoenix.tar.gz

  if [ ! -e opt/hp/hp-phoenix/srv/www/bin/nand_mnt ]; then
    fakeroot tar xzf ${scriptdir}/hp-ilo-nand_mnt-replace.tar.gz
  fi
  if [ -e opt/hp/hp-phoenix/srv/www/bin/nand_mnt -a ! -e bin/nand_mnt ]; then
    mkdir -p bin
    ln -s ../opt/hp/hp-phoenix/srv/www/bin/nand_mnt bin/nand_mnt
  fi

  chmod ugo+rx opt/hp/hp-phoenix/srv/www/bin/*nand*

  ln -s `which grep`   opt/hp/hp-phoenix/srv/www/bin/grep
  ln -s `which sdparm` opt/hp/hp-phoenix/srv/www/bin/sdparm

  mkdir -p usr/lib/python2.7/dist-packages
  cd usr/lib/python2.7/dist-packages/
  fakeroot tar xzf ${scriptdir}/${workdir}/hp-phoenix-python.tar.gz
  cd ${HPIPDEBDIR}

  simpledeb

  HPIPDEBNAM=hp-ilo-nand
  HPIPDEBDIR=${scriptdir}/${workdir}/${HPIPDEBNAM}-deb/${HPIPDEBNAM}_${HPIPDEBVER}_${HPIPDEBARCH}

  mkdir -p ${HPIPDEBDIR}

  cd ${HPIPDEBDIR}

  mkdir -p opt/hp-ilo-nand
  cd opt/hp-ilo-nand
  fakeroot tar xzf ${scriptdir}/${workdir}/hp-ilo-nand.tar.gz
  cd ${HPIPDEBDIR}

  if [ -e opt/hp-ilo-nand/bin/nand_mnt ]; then
    chmod ugo+rx opt/hp-ilo-nand/bin/*nand*

    ln -s `which grep`   opt/hp-ilo-nand/bin/grep
    ln -s `which sdparm` opt/hp-ilo-nand/bin/sdparm

    simpledeb

  else
    cd ${scriptdir}
    rm -rf ${HPIPDEBDIR}/
  fi

  cd ${scriptdir}

  sudo umount ${workdir}/mount
fi

if [ -e $gaius/sources/boot.wim ]; then
  echo "Found gaius/sources/boot.wim"
fi

